package com.nisum.producer;

import com.mongodb.spark.config.ReadConfig;

import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.functions;
import org.apache.spark.storage.StorageLevel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static com.nisum.utils.UserConstants.*;

public class SparkProducer {
    static Logger log = LoggerFactory.getLogger(SparkProducer.class);
    public static void main(String[] args)  {

        SparkSession spark = SparkSession.builder()
                .master("local")
                .appName("MongoSparkConnectorIntro")
                .config("spark.mongodb.input.uri", USERDATA_HOST)
                .getOrCreate();

        ReadConfig readConfigUserInfo = ReadConfig.create(spark)
                .withOption("uri", URI)
                .withOption("database", USER_DB)
                .withOption("collection", USERS_INFO);

        Dataset<Row> datasetUserInfo = spark.read()
                .format("mongo")
                .options(readConfigUserInfo.asOptions())
                .load();
        datasetUserInfo.createOrReplaceTempView("user");

        ReadConfig readConfigUserCardInfo = ReadConfig.create(spark)
                .withOption("uri", URI)
                .withOption("database", USER_DB)
                .withOption("collection", USERS_CARDS_INFO);

        Dataset<Row> datasetUserCardInfo = spark.read()
                .format("mongo")
                .options(readConfigUserCardInfo.asOptions())
                .load();
        datasetUserCardInfo.createOrReplaceTempView("card");
        log.debug("datasetUserInfo {}", datasetUserInfo.toJSON());
        datasetUserInfo.show();
        log.debug("datasetUserCardInfo {}", datasetUserCardInfo.toJSON());
        datasetUserCardInfo.show();

        Dataset<Row> userCardInfo = spark
                .sql("SELECT user.gender,user.regMobileNo,user.userName,card.userId as userId,card._id as cardId,card.cardNumber,card.cardStatus,card.cardType FROM user JOIN card ON user._id = card.userId");
        log.debug("userCardInfo ",datasetUserCardInfo.toJSON());
        userCardInfo.show();

        Dataset<Row> userCardInfoJson = userCardInfo.withColumn("value", functions.to_json(functions.struct(functions.col("gender"),
                functions.col("regMobileNo"), functions.col("userName"),
                functions.col("userId"), functions.col("cardId"),
                functions.col("cardNumber"), functions.col("cardStatus"),
                functions.col("cardType")))).alias("value").select("value").persist(StorageLevel.MEMORY_AND_DISK());
        userCardInfoJson.show(false);
        log.debug("userCardInfo ",userCardInfoJson.toJSON());
        userCardInfoJson
                .write()
                .format("kafka")
                .option("kafka.bootstrap.servers", KAFKA_HOST)
                .option("topic", TOPIC).save();
        spark.close();

    }
}

